@@ -1,7 +1,7 @@
 #include <uwsgi.h>
-#ifdef __linux__
+#if defined(__linux__) || defined(__FreeBSD_kernel__)
 #include <pty.h>
-#elif defined(__APPLE__) || defined(__OpenBSD__)
+#elif defined(__APPLE__) || defined(__OpenBSD__) || defined(__NetBSD__)
 #include <util.h>
 #elif defined(__FreeBSD__)
 #include <libutil.h>
@@ -21,6 +21,7 @@ struct uwsgi_pty_client {
 static struct uwsgi_pty {
 	char *addr;
 	char *remote;
+	char *uremote;
 	int queue;
 	int server_fd;
 	int master_fd;
@@ -76,7 +77,8 @@ static struct uwsgi_option uwsgi_pty_options[] = {
 	{"pty-socket", required_argument, 0, "bind the pty server on the specified address", uwsgi_opt_set_str, &upty.addr, 0},
 	{"pty-log", no_argument, 0, "send stdout/stderr to the log engine too", uwsgi_opt_true, &upty.log, 0},
 	{"pty-input", no_argument, 0, "read from original stdin in addition to pty", uwsgi_opt_true, &upty.input, 0},
-	{"pty-connect", required_argument, 0, "connect the current terminal to a pty server", uwsgi_opt_set_str, &upty.remote, 0},
+	{"pty-connect", required_argument, 0, "connect the current terminal to a pty server", uwsgi_opt_set_str, &upty.remote, UWSGI_OPT_NO_INITIAL},
+	{"pty-uconnect", required_argument, 0, "connect the current terminal to a pty server (using uwsgi protocol)", uwsgi_opt_set_str, &upty.uremote, UWSGI_OPT_NO_INITIAL},
 	{"pty-no-isig", no_argument, 0, "disable ISIG terminal attribute in client mode", uwsgi_opt_true, &upty.no_isig, 0},
 	{"pty-exec", required_argument, 0, "run the specified command soon after the pty thread is spawned", uwsgi_opt_set_str, &upty.command, 0},
 	{0, 0, 0, 0, 0, 0, 0},
@@ -256,17 +258,40 @@ static void uwsgi_pty_init() {
 
 }
 
-static int uwsgi_pty_client() {
-	if (!upty.remote) return 0;
+static void uwsgi_pty_winch() {
+	// 2 uwsgi packets
+	char uwsgi_pkt[8];
+#ifdef TIOCGWINSZ
+	struct winsize w;
+	ioctl(0, TIOCGWINSZ, &w);
+	uwsgi_pkt[0] = 0;
+	uwsgi_pkt[1] = (uint8_t) (w.ws_row & 0xff);
+        uwsgi_pkt[2] = (uint8_t) ((w.ws_row >> 8) & 0xff);
+	uwsgi_pkt[3] = 100;
+	uwsgi_pkt[4] = 0;
+	uwsgi_pkt[5] = (uint8_t) (w.ws_col & 0xff);
+        uwsgi_pkt[6] = (uint8_t) ((w.ws_col >> 8) & 0xff);
+	uwsgi_pkt[7] = 101;
+#endif
+	if (write(upty.server_fd, uwsgi_pkt, 8) != 8) {
+		uwsgi_error("uwsgi_pty_winch()/write()");
+		exit(1);
+	}
+}
 
-	uwsgi_log("[pty] connecting to %s ...\n", upty.remote);
+static int uwsgi_pty_client() {
+	if (!upty.remote && !upty.uremote) return 0;
+
+	char *remote = upty.uremote ? upty.uremote : upty.remote;
+
+	uwsgi_log("[pty] connecting to %s ...\n", remote);
 
 	// save current terminal settings
 	if (!tcgetattr(0, &uwsgi.termios)) {
         	uwsgi.restore_tc = 1;
         }
 
-	upty.server_fd = uwsgi_connect(upty.remote, uwsgi.shared->options[UWSGI_OPTION_SOCKET_TIMEOUT], 0);
+	upty.server_fd = uwsgi_connect(remote, uwsgi.shared->options[UWSGI_OPTION_SOCKET_TIMEOUT], 0);
 	if (upty.server_fd < 0) {
 		uwsgi_error("uwsgi_pty_client()/connect()");
 		exit(1);
@@ -277,8 +302,15 @@ static int uwsgi_pty_client() {
 
 	uwsgi_log("[pty] connected.\n");
 
+
 	uwsgi_pty_setterm(0);
 
+	if (upty.uremote) {
+		signal(SIGWINCH, uwsgi_pty_winch);
+		// send current terminal size
+		uwsgi_pty_winch();
+	}
+
 	upty.queue = event_queue_init();
 	event_queue_add_fd_read(upty.queue, upty.server_fd);
 	event_queue_add_fd_read(upty.queue, 0);
@@ -287,10 +319,21 @@ static int uwsgi_pty_client() {
 		char buf[8192];
 		int interesting_fd = -1;
                 int ret = event_queue_wait(upty.queue, -1, &interesting_fd);		
-		if (ret <= 0) break;
+		if (ret == 0) break;
+		if (ret < 0) {
+			if (errno == EINTR) continue;
+			break;
+		}
 		if (interesting_fd == 0) {
 			ssize_t rlen = read(0, buf, 8192);
 			if (rlen <= 0) break;
+			if (upty.uremote) {
+				struct uwsgi_header uh;
+				uh.modifier1 = 0;
+				uh.pktsize = rlen;
+				uh.modifier2 = 0;
+				if (write(upty.server_fd, &uh, 4) != 4) break;
+			}
 			if (write(upty.server_fd, buf, rlen) != rlen) break;
 			continue;
 		}	
