@@ -156,8 +156,14 @@ struct uwsgi_option uwsgi_python_options[] = {
 	{"wsgi-env-behavior", required_argument, 0, "set the strategy for allocating/deallocating the WSGI env", uwsgi_opt_set_str, &up.wsgi_env_behaviour, 0},
 	{"start_response-nodelay", no_argument, 0, "send WSGI http headers as soon as possible (PEP violation)", uwsgi_opt_true, &up.start_response_nodelay, 0},
 
+	{"wsgi-strict", no_argument, 0, "try to be fully PEP compliant disabling optimizations", uwsgi_opt_true, &up.wsgi_strict, 0},
+	{"wsgi-accept-buffer", no_argument, 0, "accept CPython buffer-compliant objects as WSGI response in addition to string/bytes", uwsgi_opt_true, &up.wsgi_accept_buffer, 0},
+	{"wsgi-accept-buffers", no_argument, 0, "accept CPython buffer-compliant objects as WSGI response in addition to string/bytes", uwsgi_opt_true, &up.wsgi_accept_buffer, 0},
+
 	{"python-version", no_argument, 0, "report python version", uwsgi_opt_pyver, NULL, UWSGI_OPT_IMMEDIATE},
 
+	{"python-raw", required_argument, 0, "load a python file for managing raw requests", uwsgi_opt_set_str, &up.raw, 0},
+
 	{0, 0, 0, 0, 0, 0, 0},
 };
 
@@ -308,6 +314,10 @@ realstuff:
 	// this time we use this higher level function
 	// as this code can be executed in a signal handler
 
+#ifdef __FreeBSD_kernel__
+	return;
+#endif
+
 	if (!Py_IsInitialized()) {
 		return;
 	}
@@ -1068,7 +1078,6 @@ void uwsgi_python_init_apps() {
         up.loaders[LOADER_CALLABLE] = uwsgi_callable_loader;
         up.loaders[LOADER_STRING_CALLABLE] = uwsgi_string_callable_loader;
 
-
 	struct uwsgi_string_list *upli = up.import_list;
 	while(upli) {
 		if (strchr(upli->value, '/') || uwsgi_endswith(upli->value, ".py")) {
@@ -1119,6 +1128,12 @@ next:
 		uppa = uppa->next;
         }
 
+	if (up.raw) {
+		up.raw_callable = uwsgi_file_loader(up.raw);
+		if (up.raw_callable) {
+			Py_INCREF(up.raw_callable);
+		}
+	}
 
 	if (up.wsgi_config != NULL) {
 		init_uwsgi_app(LOADER_UWSGI, up.wsgi_config, uwsgi.wsgi_req, up.main_thread, PYTHON_APP_TYPE_WSGI);
@@ -1354,8 +1369,10 @@ void *uwsgi_python_autoreloader_thread(void *foobar) {
 		UWSGI_RELEASE_GIL;
 		sleep(up.auto_reload);
 		UWSGI_GET_GIL;
-		// do not start monitoring til the first app is loaded (required for lazy mode)
-		if (uwsgi_apps_cnt == 0) continue;
+		if (uwsgi.lazy || uwsgi.lazy_apps) {
+			// do not start monitoring til the first app is loaded (required for lazy mode)
+			if (uwsgi_apps_cnt == 0) continue;
+		}
 #ifdef UWSGI_PYTHON_OLD
                 int pos = 0;
 #else
