@@ -5,6 +5,68 @@ extern struct uwsgi_python up;
 extern struct uwsgi_plugin python_plugin;
 extern PyTypeObject uwsgi_InputType;
 
+/*
+
+	Albeit PEP 333/3333 is clear about what kind of return object we must
+	expect from a WSGI callable, we CAN use the buffer api to optimize for lower-level
+	returns type: bytes, bytearray, array.array
+
+	to enable this optimization add --wsgi-accept-buffer
+
+	NOTE: this is a violation of the standards !!! use only if you know the implications !!!
+
+*/
+
+int uwsgi_python_send_body(struct wsgi_request *wsgi_req, PyObject *chunk) {
+	if (!up.wsgi_accept_buffer && !wsgi_req->is_raw) goto strict;
+#if defined(PYTHREE) || defined(Py_TPFLAGS_HAVE_NEWBUFFER)
+	Py_buffer pbuf;
+	int has_buffer = 0;
+#endif
+	char *content = NULL;
+	size_t content_len = 0;
+#if defined(PYTHREE) || defined(Py_TPFLAGS_HAVE_NEWBUFFER)
+	if (PyObject_CheckBuffer(chunk)) {
+		if (!PyObject_GetBuffer(chunk, &pbuf, PyBUF_SIMPLE)) {
+			content = (char *) pbuf.buf;
+			content_len = (size_t) pbuf.len;
+			has_buffer = 1;
+			goto found;
+		}
+	}
+#else
+	if (PyObject_CheckReadBuffer(chunk)) {
+		if (!PyObject_AsCharBuffer(chunk, (const char **) &content, (Py_ssize_t *) &content_len)) {
+			PyErr_Clear();
+			goto found;
+		}
+	}
+#endif
+
+strict:
+	// fallback
+	if (PyString_Check(chunk)) {
+               	content = PyString_AsString(chunk);
+               	content_len = PyString_Size(chunk);
+	}
+
+found:
+	if (content) {
+                UWSGI_RELEASE_GIL
+                uwsgi_response_write_body_do(wsgi_req, content, content_len);
+                UWSGI_GET_GIL
+#if defined(PYTHREE) || defined(Py_TPFLAGS_HAVE_NEWBUFFER)
+		if (has_buffer) PyBuffer_Release(&pbuf);
+#endif
+                uwsgi_py_check_write_errors {
+                       	uwsgi_py_write_exception(wsgi_req);
+			return -1;
+                }
+		return 1;
+	}
+	return 0;
+} 
+
 /*
 this is a hack for supporting non-file object passed to wsgi.file_wrapper
 */
@@ -169,19 +231,11 @@ int uwsgi_response_subhandler_wsgi(struct wsgi_request *wsgi_req) {
 	PyObject *pychunk;
 
 	// return or yield ?
-	if (PyString_Check((PyObject *)wsgi_req->async_result)) {
-		char *content = PyString_AsString((PyObject *)wsgi_req->async_result);
-		size_t content_len = PyString_Size((PyObject *)wsgi_req->async_result);
-		UWSGI_RELEASE_GIL
-		uwsgi_response_write_body_do(wsgi_req, content, content_len);
-		UWSGI_GET_GIL
-		uwsgi_py_check_write_errors {
-			uwsgi_py_write_exception(wsgi_req);
-		}
-		goto clear;
+	// in strict mode we do not optimize apps directly returning strings (or bytes)
+	if (!up.wsgi_strict) {
+		if (uwsgi_python_send_body(wsgi_req, (PyObject *)wsgi_req->async_result)) goto clear;
 	}
 
-
 	if (wsgi_req->sendfile_obj == wsgi_req->async_result) {
 		if (wsgi_req->sendfile_fd >= 0) {
 			UWSGI_RELEASE_GIL
@@ -222,19 +276,13 @@ exception:
 
 
 
-	if (PyString_Check(pychunk)) {
-		char *content = PyString_AsString(pychunk);
-		size_t content_len = PyString_Size(pychunk);
-		UWSGI_RELEASE_GIL
-		uwsgi_response_write_body_do(wsgi_req, content, content_len);
-		UWSGI_GET_GIL
-		uwsgi_py_check_write_errors {
-			uwsgi_py_write_exception(wsgi_req);
+	int ret = uwsgi_python_send_body(wsgi_req, pychunk);
+	if (ret != 0) {
+		if (ret < 0) {
 			Py_DECREF(pychunk);
 			goto clear;
 		}
 	}
-
 	else if (wsgi_req->sendfile_obj == pychunk) {
 		if (wsgi_req->sendfile_fd >= 0) {
 			UWSGI_RELEASE_GIL
